<?php

namespace Beau\Card\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Beau\Card\Http\Requests\CardAdminApiRequest;
use Beau\Card\Interfaces\CardRepositoryInterface;
use Beau\Card\Models\Card;

/**
 * Admin API controller class.
 */
class CardAdminApiController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.api';

    /**
     * Initialize card controller.
     *
     * @param type CardRepositoryInterface $card
     *
     * @return type
     */
    public function __construct(CardRepositoryInterface $card)
    {
        $this->middleware('api');
        $this->middleware('jwt.auth:admin.api');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $card;
        parent::__construct();
    }

    /**
     * Display a list of card.
     *
     * @return json
     */
    public function index(CardAdminApiRequest $request)
    {
        $cards  = $this->repository
            ->pushCriteria(new \Beau\Card\Repositories\Criteria\CardAdminCriteria())
            ->setPresenter('\\Beau\\Card\\Repositories\\Presenter\\CardListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $cards['code'] = 2000;
        return response()->json($cards) 
                         ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display card.
     *
     * @param Request $request
     * @param Model   Card
     *
     * @return Json
     */
    public function show(CardAdminApiRequest $request, Card $card)
    {
        $card         = $card->presenter();
        $card['code'] = 2001;
        return response()->json($card)
                         ->setStatusCode(200, 'SHOW_SUCCESS');;

    }

    /**
     * Show the form for creating a new card.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(CardAdminApiRequest $request, Card $card)
    {
        $card         = $card->presenter();
        $card['code'] = 2002;
        return response()->json($card)
                         ->setStatusCode(200, 'CREATE_SUCCESS');

    }

    /**
     * Create new card.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(CardAdminApiRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $card          = $this->repository->create($attributes);
            $card          = $card->presenter();
            $card['code']  = 2004;

            return response()->json($card)
                             ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
;
        }
    }

    /**
     * Show card for editing.
     *
     * @param Request $request
     * @param Model   $card
     *
     * @return json
     */
    public function edit(CardAdminApiRequest $request, Card $card)
    {
        $card         = $card->presenter();
        $card['code'] = 2003;
        return response()-> json($card)
                        ->setStatusCode(200, 'EDIT_SUCCESS');;
    }

    /**
     * Update the card.
     *
     * @param Request $request
     * @param Model   $card
     *
     * @return json
     */
    public function update(CardAdminApiRequest $request, Card $card)
    {
        try {

            $attributes = $request->all();

            $card->update($attributes);
            $card         = $card->presenter();
            $card['code'] = 2005;

            return response()->json($card)
                             ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the card.
     *
     * @param Request $request
     * @param Model   $card
     *
     * @return json
     */
    public function destroy(CardAdminApiRequest $request, Card $card)
    {

        try {

            $t = $card->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('card::card.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
