<?php

namespace Beau\Card\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Beau\Card\Http\Requests\CardAdminWebRequest;
use Beau\Card\Interfaces\CardRepositoryInterface;
use Beau\Card\Models\Card;

/**
 * Admin web controller class.
 */
class CardAdminController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.web';

    /**
     * Initialize card controller.
     *
     * @param type CardRepositoryInterface $card
     *
     * @return type
     */
    public function __construct(CardRepositoryInterface $card)
    {
        $this->middleware('web');
        $this->middleware('auth:admin.web');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $card;
        parent::__construct();
    }

    /**
     * Display a list of card.
     *
     * @return Response
     */
    public function index(CardAdminWebRequest $request)
    {
        $pageLimit = $request->input('pageLimit');
        if ($request->wantsJson()) {
            $cards  = $this->repository
            ->pushCriteria(new \Beau\Card\Repositories\Criteria\CardAdminCriteria())
                ->setPresenter('\\Beau\\Card\\Repositories\\Presenter\\CardListPresenter')
                ->scopeQuery(function ($query) {
                    return $query->orderBy('id', 'DESC');
                })->paginate($pageLimit);

            $cards['recordsTotal']    = $cards['meta']['pagination']['total'];
            $cards['recordsFiltered'] = $cards['meta']['pagination']['total'];
            $cards['request']         = $request->all();
            return response()->json($cards, 200);

        }

        $this   ->theme->prependTitle(trans('card::card.names').' :: ');
        return $this->theme->of('card::admin.card.index')->render();
    }

    /**
     * Display card.
     *
     * @param Request $request
     * @param Model   $card
     *
     * @return Response
     */
    public function show(CardAdminWebRequest $request, Card $card)
    {
        if (!$card->exists) {
            return response()->view('card::admin.card.new', compact('card'));
        }

        Form::populate($card);
        return response()->view('card::admin.card.show', compact('card'));
    }

    /**
     * Show the form for creating a new card.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CardAdminWebRequest $request)
    {

        $card = $this->repository->newInstance([]);

        Form::populate($card);

        return response()->view('card::admin.card.create', compact('card'));

    }

    /**
     * Create new card.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CardAdminWebRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $card          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('card::card.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/card/card/'.$card->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show card for editing.
     *
     * @param Request $request
     * @param Model   $card
     *
     * @return Response
     */
    public function edit(CardAdminWebRequest $request, Card $card)
    {
        Form::populate($card);
        return  response()->view('card::admin.card.edit', compact('card'));
    }

    /**
     * Update the card.
     *
     * @param Request $request
     * @param Model   $card
     *
     * @return Response
     */
    public function update(CardAdminWebRequest $request, Card $card)
    {
        try {

            $attributes = $request->all();

            $card->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('card::card.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/card/card/'.$card->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/card/card/'.$card->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the card.
     *
     * @param Model   $card
     *
     * @return Response
     */
    public function destroy(CardAdminWebRequest $request, Card $card)
    {

        try {

            $t = $card->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('card::card.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/card/card/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/card/card/'.$card->getRouteKey()),
            ], 400);
        }
    }
}
